@@ -1,37 +1,21 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
-#include <memory>
 #include <functional>
+#include <memory>
 
-#include "caf/fwd.hpp"
+#include "caf/detail/core_export.hpp"
+#include "caf/detail/shared_spinlock.hpp"
 #include "caf/extend.hpp"
-#include "caf/scheduled_actor.hpp"
+#include "caf/fwd.hpp"
 #include "caf/mailbox_element.hpp"
-
+#include "caf/mixin/behavior_changer.hpp"
 #include "caf/mixin/sender.hpp"
 #include "caf/mixin/subscriber.hpp"
-#include "caf/mixin/behavior_changer.hpp"
-
-#include "caf/detail/disposer.hpp"
-#include "caf/detail/shared_spinlock.hpp"
+#include "caf/scheduled_actor.hpp"
 
 namespace caf {
 
@@ -45,10 +29,13 @@ public:
 /// callback to another object, thus serving as gateway to
 /// allow any object to interact with other actors.
 /// @extends local_actor
-class actor_companion : public extend<scheduled_actor, actor_companion>::
-                               with<mixin::sender,
-                                    mixin::subscriber,
-                                    mixin::behavior_changer> {
+class CAF_CORE_EXPORT actor_companion
+  // clang-format off
+  : public extend<scheduled_actor, actor_companion>::
+           with<mixin::sender,
+                mixin::subscriber,
+                mixin::behavior_changer> {
+  // clang-format on
 public:
   // -- member types -----------------------------------------------------------
 
@@ -62,10 +49,10 @@ public:
   using lock_type = detail::shared_spinlock;
 
   /// Delegates incoming messages to user-defined event loop.
-  using enqueue_handler = std::function<void (mailbox_element_ptr)>;
+  using enqueue_handler = std::function<void(mailbox_element_ptr)>;
 
   /// Callback for actor termination.
-  using on_exit_handler = std::function<void ()>;
+  using on_exit_handler = std::function<void()>;
 
   // -- constructors, destructors ----------------------------------------------
 
@@ -75,9 +62,9 @@ public:
 
   // -- overridden functions ---------------------------------------------------
 
-  void enqueue(mailbox_element_ptr ptr, execution_unit* host) override;
+  bool enqueue(mailbox_element_ptr ptr, execution_unit* host) override;
 
-  void enqueue(strong_actor_ptr src, message_id mid, message content,
+  bool enqueue(strong_actor_ptr src, message_id mid, message content,
                execution_unit* eu) override;
 
   void launch(execution_unit* eu, bool lazy, bool hide) override;
@@ -109,4 +96,3 @@ private:
 };
 
 } // namespace caf
-
