@@ -1,35 +1,21 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
-#include <vector>
 #include <functional>
+#include <vector>
 
-#include "caf/locks.hpp"
 #include "caf/actor.hpp"
+#include "caf/detail/core_export.hpp"
+#include "caf/detail/shared_spinlock.hpp"
+#include "caf/detail/split_join.hpp"
 #include "caf/execution_unit.hpp"
+#include "caf/locks.hpp"
 #include "caf/mailbox_element.hpp"
 #include "caf/monitorable_actor.hpp"
 
-#include "caf/detail/split_join.hpp"
-#include "caf/detail/shared_spinlock.hpp"
-
 namespace caf {
 
 /// An actor poool is a lightweight abstraction for a set of workers.
@@ -37,7 +23,7 @@ namespace caf {
 /// around in an actor system to hide the actual set of workers.
 ///
 /// After construction, new workers can be added via `{'SYS', 'PUT', actor}`
-/// messages, e.g., `send(my_pool, sys_atom_v, put_atom_v, worker)`.
+/// messages, e.g., `send(my_pool, sys_atom::value, put_atom::value, worker)`.
 /// `{'SYS', 'DELETE', actor}` messages remove a specific worker from the set,
 /// `{'SYS', 'DELETE'}` removes all workers, and `{'SYS', 'GET'}` returns a
 /// `vector<actor>` containing all workers.
@@ -54,13 +40,13 @@ namespace caf {
 /// messages with as little overhead as possible, because the dispatching
 /// runs in the context of the sender.
 /// @experimental
-class actor_pool : public monitorable_actor {
+class CAF_CORE_EXPORT actor_pool : public monitorable_actor {
 public:
   using uplock = upgrade_lock<detail::shared_spinlock>;
   using actor_vec = std::vector<actor>;
-  using factory = std::function<actor ()>;
-  using policy = std::function<void (actor_system&, uplock&, const actor_vec&,
-                                     mailbox_element_ptr&, execution_unit*)>;
+  using factory = std::function<actor()>;
+  using policy = std::function<void(actor_system&, uplock&, const actor_vec&,
+                                    mailbox_element_ptr&, execution_unit*)>;
 
   /// Returns a simple round robin dispatching policy.
   static policy round_robin();
@@ -96,21 +82,26 @@ public:
 
   /// Returns an actor pool with `n` workers created by the factory
   /// function `fac` using the dispatch policy `pol`.
-  static actor make(execution_unit* eu, size_t num_workers, const factory& fac, policy pol);
+  static actor
+  make(execution_unit* eu, size_t num_workers, const factory& fac, policy pol);
 
-  void enqueue(mailbox_element_ptr what, execution_unit* eu) override;
+  bool enqueue(mailbox_element_ptr what, execution_unit* eu) override;
 
   actor_pool(actor_config& cfg);
 
   void on_destroy() override;
 
+  void setup_metrics() {
+    // nop
+  }
+
 protected:
   void on_cleanup(const error& reason) override;
 
 private:
   bool filter(upgrade_lock<detail::shared_spinlock>&,
-              const strong_actor_ptr& sender, message_id mid,
-              message_view& mv, execution_unit* eu);
+              const strong_actor_ptr& sender, message_id mid, message& msg,
+              execution_unit* eu);
 
   // call without workers_mtx_ held
   void quit(execution_unit* host);
@@ -122,4 +113,3 @@ private:
 };
 
 } // namespace caf
-
