@@ -1,38 +1,25 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
-#include <memory>
 #include <cstdint>
+#include <memory>
 #include <typeinfo>
 
+#include "caf/detail/core_export.hpp"
 #include "caf/error.hpp"
-#include "caf/optional.hpp"
-#include "caf/exit_reason.hpp"
 #include "caf/execution_unit.hpp"
+#include "caf/exit_reason.hpp"
+#include "caf/optional.hpp"
 
 namespace caf {
 
 class abstract_actor;
 
 /// Callback utility class.
-class attachable {
+class CAF_CORE_EXPORT attachable {
 public:
   attachable() = default;
   attachable(const attachable&) = delete;
@@ -89,4 +76,3 @@ public:
 using attachable_ptr = std::unique_ptr<attachable>;
 
 } // namespace caf
-
