@@ -1,21 +1,6 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright (C) 2011 - 2017                                                  *
- * Dominik Charousset <dominik.charousset (at) haw-hamburg.de>                *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
@@ -25,8 +10,7 @@
 
 #include "caf/intrusive/forward_iterator.hpp"
 
-namespace caf {
-namespace intrusive {
+namespace caf::intrusive {
 
 /// A singly-linked FIFO queue for storing tasks of varying size. This queue is
 /// used as a base type for concrete task abstractions such as `drr_queue` and
@@ -70,9 +54,7 @@ public:
   // -- constructors, destructors, and assignment operators -------------------
 
   task_queue(policy_type p)
-      : old_last_(nullptr),
-        new_head_(nullptr),
-        policy_(std::move(p)) {
+    : old_last_(nullptr), new_head_(nullptr), policy_(std::move(p)) {
     init();
   }
 
@@ -142,6 +124,15 @@ public:
       f(*i);
   }
 
+  /// Tries to find an element in the queue that matches the given predicate.
+  template <class Predicate>
+  pointer find_if(Predicate pred) {
+    for (auto i = begin(); i != end(); ++i)
+      if (pred(*i))
+        return promote(i.ptr);
+    return nullptr;
+  }
+
   // -- modifiers -------------------------------------------------------------
 
   /// Removes all elements from the queue.
@@ -174,10 +165,8 @@ public:
 
   /// @private
   task_size_type next_task_size() const noexcept {
-    if (head_.next == nullptr)
-      return 0;
-    auto ptr = promote(head_.next);
-    return policy_.task_size(*ptr);
+    auto ptr = head_.next;
+    return ptr != &tail_ ? policy_.task_size(*promote(ptr)) : 0;
   }
 
   /// @private
@@ -204,16 +193,6 @@ public:
 
   // -- iterator access --------------------------------------------------------
 
-  /// Returns an iterator to the dummy before the first element.
-  iterator before_begin() noexcept {
-    return &head_;
-  }
-
-  /// Returns an iterator to the dummy before the first element.
-  const_iterator before_begin() const noexcept {
-    return &head_;
-  }
-
   /// Returns an iterator to the dummy before the first element.
   iterator begin() noexcept {
     return head_.next;
@@ -253,6 +232,7 @@ public:
 
   /// Returns a pointer to the last element.
   pointer back() noexcept {
+    CAF_ASSERT(head_.next != &tail_);
     return promote(tail_.next);
   }
 
@@ -320,7 +300,7 @@ public:
   /// @private
   void lifo_append(node_pointer ptr) {
     if (old_last_ == nullptr) {
-      old_last_ = back();
+      old_last_ = tail_.next;
       push_back(promote(ptr));
     } else {
       ptr->next = new_head_;
@@ -383,6 +363,4 @@ protected:
   policy_type policy_;
 };
 
-} // namespace intrusive
-} // namespace caf
-
+} // namespace caf::intrusive
