@@ -1,47 +1,30 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
 #include <list>
-#include <vector>
 #include <memory>
-#include <utility>
 #include <type_traits>
-
-#include "caf/fwd.hpp"
-#include "caf/none.hpp"
-#include "caf/intrusive_ptr.hpp"
+#include <utility>
+#include <vector>
 
 #include "caf/behavior.hpp"
-#include "caf/duration.hpp"
-#include "caf/match_case.hpp"
+#include "caf/detail/behavior_impl.hpp"
+#include "caf/detail/core_export.hpp"
+#include "caf/fwd.hpp"
+#include "caf/intrusive_ptr.hpp"
 #include "caf/may_have_timeout.hpp"
 #include "caf/message.hpp"
+#include "caf/none.hpp"
 #include "caf/ref_counted.hpp"
 #include "caf/timeout_definition.hpp"
 
-#include "caf/detail/behavior_impl.hpp"
-
 namespace caf {
 
 /// A partial function implementation used to process a `message`.
-class message_handler {
+class CAF_CORE_EXPORT message_handler {
 public:
   friend class behavior;
 
@@ -55,7 +38,7 @@ public:
   using impl_ptr = intrusive_ptr<detail::behavior_impl>;
 
   /// Returns a pointer to the implementation.
-  inline const impl_ptr& as_behavior_impl() const {
+  const impl_ptr& as_behavior_impl() const {
     return impl_;
   }
 
@@ -63,7 +46,7 @@ public:
   message_handler(impl_ptr ptr);
 
   /// Checks whether the message handler is not empty.
-  inline operator bool() const {
+  operator bool() const {
     return static_cast<bool>(impl_);
   }
 
@@ -78,9 +61,10 @@ public:
   template <class... Ts>
   void assign(Ts... xs) {
     static_assert(sizeof...(Ts) > 0, "assign without arguments called");
-    static_assert(!detail::disjunction<may_have_timeout<
-                      typename std::decay<Ts>::type>::value...
-                    >::value, "Timeouts are only allowed in behaviors");
+    static_assert(
+      !detail::disjunction<
+        may_have_timeout<typename std::decay<Ts>::type>::value...>::value,
+      "Timeouts are only allowed in behaviors");
     impl_ = detail::make_behavior(xs...);
   }
 
@@ -88,37 +72,21 @@ public:
   void assign(message_handler what);
 
   /// Runs this handler and returns its (optional) result.
-  inline optional<message> operator()(message& arg) {
+  optional<message> operator()(message& arg) {
     return (impl_) ? impl_->invoke(arg) : none;
   }
 
-  /// Runs this handler and returns its (optional) result.
-  inline optional<message> operator()(type_erased_tuple& xs) {
-    return impl_ ? impl_->invoke(xs) : none;
-  }
-
   /// Runs this handler with callback.
-  inline match_case::result operator()(detail::invoke_result_visitor& f,
-                                       type_erased_tuple& xs) {
-    return impl_ ? impl_->invoke(f, xs) : match_case::no_match;
-  }
-
-  /// Runs this handler with callback.
-  inline match_case::result operator()(detail::invoke_result_visitor& f,
-                                       message& xs) {
-    return impl_ ? impl_->invoke(f, xs) : match_case::no_match;
+  bool operator()(detail::invoke_result_visitor& f, message& xs) {
+    return impl_ ? impl_->invoke(f, xs) : false;
   }
 
   /// Returns a new handler that concatenates this handler
   /// with a new handler from `xs...`.
   template <class... Ts>
-  typename std::conditional<
-    detail::disjunction<may_have_timeout<
-      typename std::decay<Ts>::type>::value...
-    >::value,
-    behavior,
-    message_handler
-  >::type
+  typename std::conditional<detail::disjunction<may_have_timeout<
+                              typename std::decay<Ts>::type>::value...>::value,
+                            behavior, message_handler>::type
   or_else(Ts&&... xs) const {
     // using a behavior is safe here, because we "cast"
     // it back to a message_handler when appropriate
@@ -133,7 +101,7 @@ public:
 
   /// @cond PRIVATE
 
-  inline message_handler& unbox() {
+  message_handler& unbox() {
     return *this;
   }
 
@@ -144,4 +112,3 @@ private:
 };
 
 } // namespace caf
-
