@@ -1,29 +1,17 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #pragma once
 
+#include <cstddef>
+#include <cstdint>
 #include <deque>
 #include <vector>
-#include <cstdint>
-#include <cstddef>
 
 #include "caf/actor_control_block.hpp"
+#include "caf/detail/core_export.hpp"
+#include "caf/detail/type_traits.hpp"
 #include "caf/downstream_msg.hpp"
 #include "caf/fwd.hpp"
 #include "caf/logger.hpp"
@@ -31,14 +19,10 @@
 #include "caf/stream_slot.hpp"
 #include "caf/system_messages.hpp"
 
-#include "caf/detail/type_traits.hpp"
-
-#include "caf/meta/type_name.hpp"
-
 namespace caf {
 
 /// State for a single path to a sink of a `downstream_manager`.
-class outbound_path {
+class CAF_CORE_EXPORT outbound_path {
 public:
   // -- member types -----------------------------------------------------------
 
@@ -65,11 +49,6 @@ public:
 
   // -- downstream communication -----------------------------------------------
 
-  /// Sends an `open_stream_msg` handshake.
-  static void emit_open(local_actor* self, stream_slot slot,
-                        strong_actor_ptr to, message handshake_data,
-                        stream_priority prio);
-
   /// Sends a `downstream_msg::batch` on this path. Decrements `open_credit` by
   /// `xs_size` and increments `next_batch_id` by 1.
   void emit_batch(local_actor* self, int32_t xs_size, message xs);
@@ -101,18 +80,19 @@ public:
   /// Calls `emit_batch` for each chunk in the cache, whereas each chunk is of
   /// size `desired_batch_size`. Does nothing for pending paths.
   template <class T>
-  void emit_batches(local_actor* self, std::vector<T>& cache,
-                    bool force_underfull) {
+  void
+  emit_batches(local_actor* self, std::vector<T>& cache, bool force_underfull) {
     CAF_LOG_TRACE(CAF_ARG(slots) << CAF_ARG(open_credit) << CAF_ARG(cache)
-                  << CAF_ARG(force_underfull));
+                                 << CAF_ARG(force_underfull));
     if (pending())
       return;
     CAF_ASSERT(open_credit >= 0);
     CAF_ASSERT(desired_batch_size > 0);
-    CAF_ASSERT(cache.size() <= std::numeric_limits<int32_t>::max());
+    CAF_ASSERT(cache.size()
+               <= static_cast<size_t>(std::numeric_limits<int32_t>::max()));
     auto first = cache.begin();
-    auto last = first + std::min(open_credit,
-                                 static_cast<int32_t>(cache.size()));
+    auto last
+      = first + std::min(open_credit, static_cast<int32_t>(cache.size()));
     if (first == last)
       return;
     auto i = emit_batches_impl(self, first, last, force_underfull);
@@ -130,9 +110,9 @@ public:
   void emit_irregular_shutdown(local_actor* self, error reason);
 
   /// Sends a `downstream_msg::forced_close`.
-  static void emit_irregular_shutdown(local_actor* self, stream_slots slots,
-                                      const strong_actor_ptr& hdl,
-                                      error reason);
+  static void
+  emit_irregular_shutdown(local_actor* self, stream_slots slots,
+                          const strong_actor_ptr& hdl, error reason);
 
   // -- properties -------------------------------------------------------------
 
@@ -170,9 +150,6 @@ public:
   /// ACKs, i.e., receiving an ACK with a higher ID is not an error.
   int64_t next_ack_id;
 
-  /// Stores the maximum capacity of the downstream actor.
-  int32_t max_capacity;
-
   /// Stores whether an outbound path is marked for removal. The
   /// `downstream_manger` no longer sends new batches to a closing path, but
   /// buffered batches are still shipped. The owning `stream_manager` removes
@@ -183,10 +160,12 @@ public:
 
 /// @relates outbound_path
 template <class Inspector>
-typename Inspector::result_type inspect(Inspector& f, outbound_path& x) {
-  return f(meta::type_name("outbound_path"), x.slots, x.hdl, x.next_batch_id,
-           x.open_credit, x.desired_batch_size, x.next_ack_id);
+bool inspect(Inspector& f, outbound_path& x) {
+  return f.object(x).fields(f.field("slots", x.slots), f.field("hdl", x.hdl),
+                            f.field("next_batch_id", x.next_batch_id),
+                            f.field("open_credit", x.open_credit),
+                            f.field("desired_batch_size", x.desired_batch_size),
+                            f.field("next_ack_id", x.next_ack_id));
 }
 
 } // namespace caf
-
