@@ -1,26 +1,15 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #include "caf/ipv4_address.hpp"
 
+#include <cstring>
+
 #include "caf/detail/network_order.hpp"
 #include "caf/detail/parser/read_ipv4_address.hpp"
 #include "caf/error.hpp"
+#include "caf/message.hpp"
 #include "caf/parser_state.hpp"
 #include "caf/pec.hpp"
 #include "caf/string_view.hpp"
@@ -56,6 +45,11 @@ ipv4_address::ipv4_address() {
 ipv4_address::ipv4_address(array_type bytes) {
   memcpy(bytes_.data(), bytes.data(), bytes.size());
 }
+// -- comparison ---------------------------------------------------------------
+
+int ipv4_address::compare(ipv4_address other) const noexcept {
+  return memcmp(bytes().data(), other.bytes().data(), num_bytes);
+}
 
 // -- properties ---------------------------------------------------------------
 
