@@ -1,20 +1,6 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #include "caf/outbound_path.hpp"
 
@@ -34,14 +20,13 @@ constexpr int32_t max_batch_size = 128 * 1024;
 
 outbound_path::outbound_path(stream_slot sender_slot,
                              strong_actor_ptr receiver_hdl)
-    : slots(sender_slot, invalid_stream_slot),
-      hdl(std::move(receiver_hdl)),
-      next_batch_id(1),
-      open_credit(0),
-      desired_batch_size(50),
-      next_ack_id(1),
-      max_capacity(0),
-      closing(false) {
+  : slots(sender_slot, invalid_stream_slot),
+    hdl(std::move(receiver_hdl)),
+    next_batch_id(1),
+    open_credit(0),
+    desired_batch_size(50),
+    next_ack_id(1),
+    closing(false) {
   // nop
 }
 
@@ -49,22 +34,6 @@ outbound_path::~outbound_path() {
   // nop
 }
 
-void outbound_path::emit_open(local_actor* self, stream_slot slot,
-                              strong_actor_ptr to, message handshake_data,
-                              stream_priority prio) {
-  CAF_LOG_TRACE(CAF_ARG(slot) << CAF_ARG(to) << CAF_ARG(handshake_data)
-                << CAF_ARG(prio));
-  CAF_ASSERT(self != nullptr);
-  CAF_ASSERT(to != nullptr);
-  // Make sure we receive errors from this point on.
-  stream_aborter::add(to, self->address(), slot,
-                      stream_aborter::sink_aborter);
-  // Send message.
-  unsafe_send_as(self, to,
-                 open_stream_msg{slot, std::move(handshake_data), self->ctrl(),
-                                 nullptr, prio});
-}
-
 void outbound_path::emit_batch(local_actor* self, int32_t xs_size, message xs) {
   CAF_LOG_TRACE(CAF_ARG(slots) << CAF_ARG(xs_size) << CAF_ARG(xs));
   CAF_ASSERT(xs_size > 0);
