@@ -1,21 +1,6 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright (C) 2011 - 2017                                                  *
- * Dominik Charousset <dominik.charousset (at) haw-hamburg.de>                *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 // This test simulates a complex multiplexing over multiple layers of WDRR
 // scheduled queues. The goal is to reduce the complex mailbox management of
@@ -31,6 +16,8 @@
 
 #define CAF_SUITE native_streaming_classes
 
+#include "core-test.hpp"
+
 #include <memory>
 #include <numeric>
 
@@ -38,12 +25,29 @@
 #include "caf/actor_system_config.hpp"
 #include "caf/broadcast_downstream_manager.hpp"
 #include "caf/buffered_downstream_manager.hpp"
+#include "caf/detail/gcd.hpp"
+#include "caf/detail/overload.hpp"
+#include "caf/detail/stream_sink_impl.hpp"
+#include "caf/detail/stream_source_impl.hpp"
+#include "caf/detail/stream_stage_impl.hpp"
 #include "caf/downstream_manager.hpp"
 #include "caf/downstream_msg.hpp"
 #include "caf/inbound_path.hpp"
+#include "caf/intrusive/drr_queue.hpp"
+#include "caf/intrusive/singly_linked.hpp"
+#include "caf/intrusive/task_result.hpp"
+#include "caf/intrusive/wdrr_dynamic_multiplexed_queue.hpp"
+#include "caf/intrusive/wdrr_fixed_multiplexed_queue.hpp"
 #include "caf/mailbox_element.hpp"
+#include "caf/mixin/sender.hpp"
 #include "caf/no_stages.hpp"
 #include "caf/outbound_path.hpp"
+#include "caf/policy/arg.hpp"
+#include "caf/policy/categorized.hpp"
+#include "caf/policy/downstream_messages.hpp"
+#include "caf/policy/normal_messages.hpp"
+#include "caf/policy/upstream_messages.hpp"
+#include "caf/policy/urgent_messages.hpp"
 #include "caf/scheduled_actor.hpp"
 #include "caf/send.hpp"
 #include "caf/stream_manager.hpp"
@@ -55,39 +59,6 @@
 #include "caf/upstream_msg.hpp"
 #include "caf/variant.hpp"
 
-#include "caf/scheduler/test_coordinator.hpp"
-
-#include "caf/policy/arg.hpp"
-#include "caf/policy/categorized.hpp"
-#include "caf/policy/downstream_messages.hpp"
-#include "caf/policy/normal_messages.hpp"
-#include "caf/policy/upstream_messages.hpp"
-#include "caf/policy/urgent_messages.hpp"
-
-#include "caf/mixin/sender.hpp"
-
-#include "caf/test/unit_test.hpp"
-
-#include "caf/intrusive/drr_queue.hpp"
-#include "caf/intrusive/singly_linked.hpp"
-#include "caf/intrusive/task_result.hpp"
-#include "caf/intrusive/wdrr_dynamic_multiplexed_queue.hpp"
-#include "caf/intrusive/wdrr_fixed_multiplexed_queue.hpp"
-
-#include "caf/detail/gcd.hpp"
-#include "caf/detail/overload.hpp"
-#include "caf/detail/stream_sink_impl.hpp"
-#include "caf/detail/stream_source_impl.hpp"
-#include "caf/detail/stream_stage_impl.hpp"
-#include "caf/detail/tick_emitter.hpp"
-
-CAF_BEGIN_TYPE_ID_BLOCK(native_streaming_classes, first_custom_type_id)
-
-  CAF_ADD_TYPE_ID(native_streaming_classes, (caf::stream<int32_t>) )
-  CAF_ADD_TYPE_ID(native_streaming_classes, (std::vector<int32_t>) )
-
-CAF_END_TYPE_ID_BLOCK(native_streaming_classes)
-
 using std::vector;
 
 using namespace caf;
@@ -100,7 +71,7 @@ namespace {
 struct print_with_comma_t {
   bool first = true;
   template <class T>
-  std::ostream&  operator()(std::ostream& out, const T& x) {
+  std::ostream& operator()(std::ostream& out, const T& x) {
     if (!first)
       out << ", ";
     else
@@ -119,8 +90,8 @@ std::string collapse_args(const T& x, const Ts&... xs) {
 }
 
 #define TRACE(name, type, ...)                                                 \
-  CAF_MESSAGE(name << " received a " << #type << ": "                          \
-                   << collapse_args(__VA_ARGS__));
+  MESSAGE(name << " received a " << #type << ": "                              \
+               << collapse_args(__VA_ARGS__));
 
 const char* name_of(const strong_actor_ptr& x) {
   CAF_ASSERT(x != nullptr);
@@ -137,7 +108,7 @@ const char* name_of(const actor_addr& x) {
 using mboxqueue = scheduled_actor::mailbox_policy::queue_type;
 
 template <size_t Value>
-using uint_constant =  std::integral_constant<size_t, Value>;
+using uint_constant = std::integral_constant<size_t, Value>;
 
 using urgent_async_id = uint_constant<scheduled_actor::urgent_queue_index>;
 
@@ -162,13 +133,10 @@ public:
   /// Defines the container for storing message handlers.
   using behavior_type = behavior;
 
-  /// The type of a single tick.
-  using clock_type = detail::tick_emitter::clock_type;
-
   /// The type of a single tick.
   using time_point = clock_type::time_point;
 
-  /// Difference between two points in time.
+  /// A time interval in the resolution of the actor clock.
   using duration_type = time_point::duration;
 
   /// The type of a single tick.
@@ -176,27 +144,19 @@ public:
 
   // -- constructors, destructors, and assignment operators --------------------
 
-  entity(actor_config& cfg, const char* cstr_name, time_point* global_time,
-         duration_type credit_interval, duration_type force_batches_interval)
-      : super(cfg),
-        mbox(unit, unit, unit, unit, unit),
-        name_(cstr_name),
-        global_time_(global_time),
-        tick_emitter_(global_time == nullptr ? clock_type::now()
-                                             : *global_time) {
-    auto cycle = detail::gcd(credit_interval.count(),
-                             force_batches_interval.count());
-    ticks_per_force_batches_interval =
-      static_cast<size_t>(force_batches_interval.count() / cycle);
-    ticks_per_credit_interval =
-      static_cast<size_t>(credit_interval.count() / cycle);
-    tick_emitter_.interval(duration_type{cycle});
+  entity(actor_config& cfg, const char* cstr_name, time_point* global_time)
+    : super(cfg),
+      mbox(unit, unit, unit, unit, unit),
+      name_(cstr_name),
+      global_time_(global_time) {
+    CAF_ASSERT(global_time_ != nullptr);
   }
 
-  void enqueue(mailbox_element_ptr what, execution_unit*) override {
+  bool enqueue(mailbox_element_ptr what, execution_unit*) override {
     auto push_back_result = mbox.push_back(std::move(what));
-    CAF_CHECK_EQUAL(push_back_result, true);
+    CHECK_EQ(push_back_result, true);
     CAF_ASSERT(push_back_result);
+    return true;
   }
 
   void attach(attachable_ptr) override {
@@ -235,9 +195,9 @@ public:
     return nullptr;
   }
 
-  void start_streaming(entity& ref, int32_t num_messages) {
+  void start_streaming(entity& ref, int num_messages) {
     CAF_REQUIRE_NOT_EQUAL(num_messages, 0);
-    using downstream_manager = broadcast_downstream_manager<int32_t>;
+    using downstream_manager = broadcast_downstream_manager<int>;
     struct driver final : public stream_source_driver<downstream_manager> {
     public:
       driver(int32_t sentinel) : x_(0), sentinel_(sentinel) {
@@ -253,14 +213,15 @@ public:
       bool done() const noexcept override {
         return x_ == sentinel_;
       }
+
     private:
       int32_t x_;
       int32_t sentinel_;
     };
     auto mgr = detail::make_stream_source<driver>(this, num_messages);
     auto res = mgr->add_outbound_path(ref.ctrl());
-    CAF_MESSAGE(name_ << " starts streaming to " << ref.name()
-                << " on slot " << res.value());
+    MESSAGE(name_ << " starts streaming to " << ref.name() << " on slot "
+                  << res.value());
   }
 
   void forward_to(entity& ref) {
@@ -269,13 +230,13 @@ public:
     public:
       using super = stream_stage_driver<int32_t, downstream_manager>;
 
-      driver(downstream_manager& out, vector<int32_t>* log)
-        : super(out),
-          log_(log) {
+      driver(downstream_manager& out, vector<int32_t>* log, const char* name)
+        : super(out), log_(log), name(name) {
         // nop
       }
 
       void process(downstream<int>& out, vector<int>& batch) override {
+        MESSAGE(name << " forwards " << batch.size() << " elements");
         log_->insert(log_->end(), batch.begin(), batch.end());
         out.append(batch.begin(), batch.end());
       }
@@ -286,11 +247,12 @@ public:
 
     private:
       vector<int>* log_;
+      const char* name;
     };
-    forwarder = detail::make_stream_stage<driver>(this, &data);
+    forwarder = detail::make_stream_stage<driver>(this, &data, name_);
     auto res = forwarder->add_outbound_path(ref.ctrl());
-    CAF_MESSAGE(name_ << " starts forwarding to " << ref.name()
-                << " on slot " << res.value());
+    MESSAGE(name_ << " starts forwarding to " << ref.name() << " on slot "
+                  << res.value());
   }
 
   void operator()(open_stream_msg& hs) {
@@ -309,6 +271,7 @@ public:
         void process(std::vector<int>& xs) override {
           log_->insert(log_->end(), xs.begin(), xs.end());
         }
+
       private:
         vector<int>* log_;
       };
@@ -329,46 +292,26 @@ public:
 
   void operator()(stream_slots slots, actor_addr& sender,
                   upstream_msg::ack_batch& x) {
-    TRACE(name_, ack_batch, CAF_ARG(slots),
-          CAF_ARG2("sender", name_of(sender)), CAF_ARG(x));
+    TRACE(name_, ack_batch, CAF_ARG(slots), CAF_ARG2("sender", name_of(sender)),
+          CAF_ARG(x));
     scheduled_actor::handle_upstream_msg(slots, sender, x);
   }
 
-  void advance_time() {
-    auto cycle = std::chrono::milliseconds(100);
-    auto desired_batch_complexity = std::chrono::microseconds(50);
-    auto f = [&](tick_type x) {
-      if (x % ticks_per_force_batches_interval == 0) {
-        // Force batches on all output paths.
-        for (auto& kvp : stream_managers())
-          kvp.second->out().force_emit_batches();
-      }
-      if (x % ticks_per_credit_interval == 0) {
-        // Fill credit on each input path up to 30.
-        auto& qs = get<dmsg_id::value>(mbox.queues()).queues();
-        for (auto& kvp : qs) {
-          auto inptr = kvp.second.policy().handler.get();
-          auto bs = static_cast<int32_t>(kvp.second.total_task_size());
-          inptr->emit_ack_batch(this, bs, now(), cycle,
-                                desired_batch_complexity);
-        }
-      }
-    };
-    tick_emitter_.update(now(), f);
+  void tick() {
+    for (auto& kvp : stream_managers())
+      kvp.second->tick(now());
   }
 
-  inbound_path* make_inbound_path(stream_manager_ptr mgr, stream_slots slots,
-                                  strong_actor_ptr sender,
-                                  rtti_pair rtti) override {
+  virtual bool add_inbound_path(type_id_t,
+                                std::unique_ptr<inbound_path> path) override {
     using policy_type = policy::downstream_messages::nested;
     auto res = get<dmsg_id::value>(mbox.queues())
-               .queues().emplace(slots.receiver, policy_type{nullptr});
+                 .queues()
+                 .emplace(path->slots.receiver, policy_type{nullptr});
     if (!res.second)
-      return nullptr;
-    auto path = new inbound_path(std::move(mgr), slots, std::move(sender),
-                                 rtti);
-    res.first->second.policy().handler.reset(path);
-    return path;
+      return false;
+    res.first->second.policy().handler = std::move(path);
+    return true;
   }
 
   void erase_inbound_path_later(stream_slot slot) override {
@@ -390,7 +333,14 @@ public:
   }
 
   time_point now() {
-    return global_time_ == nullptr ? clock_type::now() : *global_time_;
+    return *global_time_;
+  }
+
+  void push() {
+    if (forwarder)
+      forwarder->push();
+    for (auto mgr : active_stream_managers())
+      mgr->push();
   }
 
   // -- member variables -------------------------------------------------------
@@ -403,7 +353,6 @@ public:
   tick_type ticks_per_force_batches_interval;
   tick_type ticks_per_credit_interval;
   time_point* global_time_;
-  detail::tick_emitter tick_emitter_;
 };
 
 struct msg_visitor {
@@ -421,8 +370,7 @@ struct msg_visitor {
 
   result_type operator()(normal_async_id, entity::normal_queue&,
                          mailbox_element& x) {
-    CAF_REQUIRE_EQUAL(x.content().type_token(),
-                      make_type_token<open_stream_msg>());
+    CAF_REQUIRE(x.content().match_elements<open_stream_msg>());
     self->current_mailbox_element(&x);
     (*self)(x.content().get_mutable_as<open_stream_msg>(0));
     self->current_mailbox_element(nullptr);
@@ -430,32 +378,28 @@ struct msg_visitor {
   }
 
   result_type operator()(umsg_id, entity::upstream_queue&, mailbox_element& x) {
-    CAF_REQUIRE(x.content().type_token() == make_type_token<upstream_msg>());
+    CAF_REQUIRE(x.content().match_elements<upstream_msg>());
     self->current_mailbox_element(&x);
     auto& um = x.content().get_mutable_as<upstream_msg>(0);
     auto f = detail::make_overload(
-      [&](upstream_msg::ack_open& y) {
-        (*self)(um.slots, um.sender, y);
-      },
-      [&](upstream_msg::ack_batch& y) {
-        (*self)(um.slots, um.sender, y);
-      },
+      [&](upstream_msg::ack_open& y) { (*self)(um.slots, um.sender, y); },
+      [&](upstream_msg::ack_batch& y) { (*self)(um.slots, um.sender, y); },
       [](upstream_msg::drop&) {
         CAF_FAIL("did not expect upstream_msg::drop");
       },
       [](upstream_msg::forced_drop&) {
         CAF_FAIL("did not expect upstream_msg::forced_drop");
-      }
-    );
+      });
     visit(f, um.content);
     self->current_mailbox_element(nullptr);
+    self->push();
     return intrusive::task_result::resume;
   }
 
   result_type operator()(dmsg_id, entity::downstream_queue& qs, stream_slot,
                          policy::downstream_messages::nested_queue_type& q,
                          mailbox_element& x) {
-    CAF_REQUIRE(x.content().type_token() == make_type_token<downstream_msg>());
+    CAF_REQUIRE(x.content().match_elements<downstream_msg>());
     self->current_mailbox_element(&x);
     auto inptr = q.policy().handler.get();
     if (inptr == nullptr)
@@ -463,10 +407,10 @@ struct msg_visitor {
     auto& dm = x.content().get_mutable_as<downstream_msg>(0);
     auto f = detail::make_overload(
       [&](downstream_msg::batch& y) {
+        TRACE(self->name(), batch, CAF_ARG(dm.slots), CAF_ARG(y.xs_size));
         inptr->handle(y);
         if (inptr->mgr->done()) {
-          CAF_MESSAGE(self->name()
-                      << " is done receiving and closes its manager");
+          MESSAGE(self->name() << " is done receiving and closes its manager");
           inptr->mgr->stop();
         }
         return intrusive::task_result::resume;
@@ -508,15 +452,7 @@ struct msg_visitor {
 struct fixture {
   using scheduler_type = scheduler::test_coordinator;
 
-  struct timing_config {
-    timespan credit_interval = std::chrono::milliseconds(100);
-
-    timespan force_batches_interval = std::chrono::milliseconds(50);
-
-    timespan step = force_batches_interval;
-  };
-
-  timing_config tc;
+  timespan max_batch_delay = defaults::stream::max_batch_delay;
 
   actor_system_config cfg;
   actor_system sys;
@@ -529,13 +465,11 @@ struct fixture {
   entity& bob;
   entity& carl;
 
-  static actor spawn(actor_system& sys, actor_id id, const char* name,
-                     timing_config& tc) {
+  static actor spawn(actor_system& sys, actor_id id, const char* name) {
     actor_config conf;
     auto& clock = dynamic_cast<scheduler_type&>(sys.scheduler()).clock();
     auto global_time = &clock.current_time;
-    return make_actor<entity>(id, node_id{}, &sys, conf, name, global_time,
-                              tc.credit_interval, tc.force_batches_interval);
+    return make_actor<entity>(id, node_id{}, &sys, conf, name, global_time);
   }
 
   static entity& fetch(const actor& hdl) {
@@ -546,31 +480,32 @@ struct fixture {
     if (auto err = cfg.parse(caf::test::engine::argc(),
                              caf::test::engine::argv()))
       CAF_FAIL("parsing the config failed: " << to_string(err));
-    cfg.set("scheduler.policy", caf::atom("testing"));
-    cfg.add_message_types<id_block::native_streaming_classes>();
+    cfg.set("caf.scheduler.policy", "testing");
+    cfg.set("caf.stream.credit-policy", "token-based");
+    cfg.set("caf.stream.token-based-policy.batch-size", 50);
+    cfg.set("caf.stream.token-based-policy.buffer-size", 200);
     return cfg;
   }
 
   fixture()
-      : sys(init_config(cfg)),
-        sched(dynamic_cast<scheduler_type&>(sys.scheduler())),
-        alice_hdl(spawn(sys, 0, "alice", tc)),
-        bob_hdl(spawn(sys, 1, "bob", tc)),
-        carl_hdl(spawn(sys, 2, "carl", tc)),
-        alice(fetch(alice_hdl)),
-        bob(fetch(bob_hdl)),
-        carl(fetch(carl_hdl)) {
-    // Configure the clock to measure each batch item with 1us.
-    sched.clock().time_per_unit.emplace(atom("batch"), timespan{1000});
+    : sys(init_config(cfg)),
+      sched(dynamic_cast<scheduler_type&>(sys.scheduler())),
+      alice_hdl(spawn(sys, 0, "alice")),
+      bob_hdl(spawn(sys, 1, "bob")),
+      carl_hdl(spawn(sys, 2, "carl")),
+      alice(fetch(alice_hdl)),
+      bob(fetch(bob_hdl)),
+      carl(fetch(carl_hdl)) {
+    // nop
   }
 
   ~fixture() {
     // Check whether all actors cleaned up their state properly.
     entity* xs[] = {&alice, &bob, &carl};
     for (auto x : xs) {
-      CAF_CHECK(get<dmsg_id::value>(x->mbox.queues()).queues().empty());
-      CAF_CHECK(x->pending_stream_managers().empty());
-      CAF_CHECK(x->stream_managers().empty());
+      CHECK(get<dmsg_id::value>(x->mbox.queues()).queues().empty());
+      CHECK(x->pending_stream_managers().empty());
+      CHECK(x->stream_managers().empty());
     }
   }
 
@@ -586,10 +521,10 @@ struct fixture {
   template <class... Ts>
   void next_cycle(Ts&... xs) {
     entity* es[] = {&xs...};
-    CAF_MESSAGE("advance clock by " << tc.credit_interval.count() << "ns");
-    sched.clock().current_time += tc.credit_interval;
+    MESSAGE("advance clock by " << max_batch_delay);
+    sched.clock().current_time += max_batch_delay;
     for (auto e : es)
-      e->advance_time();
+      e->tick();
   }
 
   template <class F, class... Ts>
@@ -601,12 +536,11 @@ struct fixture {
       while (!std::all_of(std::begin(fs), std::end(fs), mailbox_empty))
         for (auto& f : fs)
           f.self->mbox.new_round(1, f);
-      CAF_MESSAGE("advance clock by " << tc.step.count() << "ns");
-      sched.clock().current_time += tc.step;
+      MESSAGE("advance clock by " << max_batch_delay);
+      sched.clock().current_time += max_batch_delay;
       for (auto e : es)
-        e->advance_time();
-    }
-    while (!pred());
+        e->tick();
+    } while (!pred());
   }
 
   bool done_streaming() {
@@ -628,56 +562,46 @@ vector<int> make_iota(int first, int last) {
 
 // -- unit tests ---------------------------------------------------------------
 
-CAF_TEST_FIXTURE_SCOPE(native_streaming_classes_tests, fixture)
+BEGIN_FIXTURE_SCOPE(fixture)
 
 CAF_TEST(depth_2_pipeline_30_items) {
   alice.start_streaming(bob, 30);
-  loop(alice, bob);
-  next_cycle(alice, bob); // emit first ack_batch
-  loop(alice, bob);
-  next_cycle(alice, bob); // to emit final ack_batch
-  loop(alice, bob);
-  CAF_CHECK_EQUAL(bob.data, make_iota(0, 30));
+  loop_until([&] { return done_streaming(); }, alice, bob);
+  CHECK_EQ(bob.data, make_iota(0, 30));
 }
 
-CAF_TEST(depth_2_pipeline_2000_items) {
-  constexpr size_t num_messages = 2000;
+CAF_TEST(depth_2_pipeline_500_items) {
+  constexpr size_t num_messages = 500;
   alice.start_streaming(bob, num_messages);
   loop_until([&] { return done_streaming(); }, alice, bob);
-  CAF_CHECK_EQUAL(bob.data, make_iota(0, num_messages));
+  CHECK_EQ(bob.data, make_iota(0, num_messages));
 }
 
 CAF_TEST(depth_3_pipeline_30_items) {
   bob.forward_to(carl);
   alice.start_streaming(bob, 30);
-  loop(alice, bob, carl);
-  next_cycle(alice, bob, carl); // emit first ack_batch
-  loop(alice, bob, carl);
-  next_cycle(alice, bob, carl);
-  loop(alice, bob, carl);
-  next_cycle(alice, bob, carl); // emit final ack_batch
-  loop(alice, bob, carl);
-  CAF_CHECK_EQUAL(bob.data, make_iota(0, 30));
-  CAF_CHECK_EQUAL(carl.data, make_iota(0, 30));
+  loop_until([&] { return done_streaming(); }, alice, bob, carl);
+  CHECK_EQ(bob.data, make_iota(0, 30));
+  CHECK_EQ(carl.data, make_iota(0, 30));
 }
 
-CAF_TEST(depth_3_pipeline_2000_items) {
-  constexpr size_t num_messages = 2000;
+CAF_TEST(depth_3_pipeline_500_items) {
+  constexpr size_t num_messages = 500;
   bob.forward_to(carl);
   alice.start_streaming(bob, num_messages);
-  CAF_MESSAGE("loop over alice and bob until bob is congested");
+  MESSAGE("loop over alice and bob until bob is congested");
   loop(alice, bob);
-  CAF_CHECK_NOT_EQUAL(bob.data.size(), 0u);
-  CAF_CHECK_EQUAL(carl.data.size(), 0u);
-  CAF_MESSAGE("loop over bob and carl until bob finsihed sending");
+  CHECK_NE(bob.data.size(), 0u);
+  CHECK_EQ(carl.data.size(), 0u);
+  MESSAGE("loop over bob and carl until bob finished sending");
   // bob has one batch from alice in its mailbox that bob will read when
   // becoming uncongested again
   loop(bob, carl);
-  CAF_CHECK_EQUAL(bob.data.size(), carl.data.size());
-  CAF_MESSAGE("loop over all until done");
+  CHECK_EQ(bob.data.size(), carl.data.size());
+  MESSAGE("loop over all until done");
   loop_until([&] { return done_streaming(); }, alice, bob, carl);
-  CAF_CHECK_EQUAL(bob.data, make_iota(0, num_messages));
-  CAF_CHECK_EQUAL(carl.data, make_iota(0, num_messages));
+  CHECK_EQ(bob.data, make_iota(0, num_messages));
+  CHECK_EQ(carl.data, make_iota(0, num_messages));
 }
 
-CAF_TEST_FIXTURE_SCOPE_END()
+END_FIXTURE_SCOPE()
