@@ -1,26 +1,10 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
-
-#include "caf/after.hpp"
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #define CAF_SUITE request_timeout
 
-#include "caf/test/dsl.hpp"
+#include "core-test.hpp"
 
 #include <chrono>
 
@@ -28,22 +12,17 @@
 
 using namespace caf;
 
-using std::string;
 using std::chrono::milliseconds;
 using std::chrono::seconds;
 
+using namespace std::string_literals;
+
 namespace {
 
-using ping_atom = atom_constant<atom("ping")>;
-using pong_atom = atom_constant<atom("pong")>;
-using timeout_atom = atom_constant<atom("timeout")>;
-
 struct pong_state {
-  static const char* name;
+  static inline const char* name = "pong";
 };
 
-const char* pong_state::name = "pong";
-
 behavior pong(stateful_actor<pong_state>*) {
   return {
     [=](ping_atom) { return pong_atom_v; },
@@ -51,16 +30,14 @@ behavior pong(stateful_actor<pong_state>*) {
 }
 
 struct ping_state {
-  static const char* name;
+  static inline const char* name = "ping";
   bool had_first_timeout = false; // unused in ping_singleN functions
 };
 
-const char* ping_state::name = "ping";
-
 using ping_actor = stateful_actor<ping_state>;
 
 using fptr = behavior (*)(ping_actor*, bool*, const actor&);
-using test_vec = std::vector<std::pair<fptr, string>>;
+using test_vec = std::vector<std::pair<fptr, std::string>>;
 
 // assumes to receive a timeout (sent via delayed_send) before pong replies
 behavior ping_single1(ping_actor* self, bool* had_timeout, const actor& buddy) {
@@ -82,6 +59,7 @@ behavior ping_single2(ping_actor* self, bool* had_timeout, const actor& buddy) {
     [=](pong_atom) { CAF_FAIL("received pong atom"); },
     after(std::chrono::seconds(1)) >>
       [=] {
+        CAF_LOG_TRACE(CAF_ARG2("had_timeout", *had_timeout));
         *had_timeout = true;
         self->quit();
       },
@@ -109,7 +87,7 @@ behavior ping_nested1(ping_actor* self, bool* had_timeout, const actor& buddy) {
     [=](timeout_atom) {
       self->state.had_first_timeout = true;
       self->become(after(milliseconds(100)) >> [=] {
-        CAF_CHECK(self->state.had_first_timeout);
+        CHECK(self->state.had_first_timeout);
         *had_timeout = true;
         self->quit();
       });
@@ -127,7 +105,7 @@ behavior ping_nested2(ping_actor* self, bool* had_timeout, const actor& buddy) {
       [=] {
         self->state.had_first_timeout = true;
         self->become(after(milliseconds(100)) >> [=] {
-          CAF_CHECK(self->state.had_first_timeout);
+          CHECK(self->state.had_first_timeout);
           *had_timeout = true;
           self->quit();
         });
@@ -151,7 +129,7 @@ behavior ping_nested3(ping_actor* self, bool* had_timeout, const actor& buddy) {
   return {
     after(milliseconds(100)) >>
       [=] {
-        CAF_CHECK(self->state.had_first_timeout);
+        CHECK(self->state.had_first_timeout);
         *had_timeout = true;
         self->quit();
       },
@@ -232,7 +210,7 @@ behavior ping_multiplexed3(ping_actor* self, bool* had_timeout,
 
 } // namespace
 
-CAF_TEST_FIXTURE_SCOPE(request_timeout_tests, test_coordinator_fixture<>)
+BEGIN_FIXTURE_SCOPE(test_coordinator_fixture<>)
 
 CAF_TEST(single_timeout) {
   test_vec fs{{ping_single1, "ping_single1"},
@@ -240,19 +218,19 @@ CAF_TEST(single_timeout) {
               {ping_single3, "ping_single3"}};
   for (auto f : fs) {
     bool had_timeout = false;
-    CAF_MESSAGE("test implemenation " << f.second);
+    MESSAGE("test implementation " << f.second);
     auto testee = sys.spawn(f.first, &had_timeout, sys.spawn<lazy_init>(pong));
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"ping"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "ping"s);
     sched.run_once();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"pong"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "pong"s);
     sched.trigger_timeout();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 2u);
     // now, the timeout message is already dispatched, while pong did
     // not respond to the message yet, i.e., timeout arrives before response
-    CAF_CHECK_EQUAL(sched.run(), 2u);
-    CAF_CHECK(had_timeout);
+    CHECK_EQ(sched.run(), 2u);
+    CHECK(had_timeout);
   }
 }
 
@@ -262,13 +240,13 @@ CAF_TEST(nested_timeout) {
               {ping_nested3, "ping_nested3"}};
   for (auto f : fs) {
     bool had_timeout = false;
-    CAF_MESSAGE("test implemenation " << f.second);
+    MESSAGE("test implementation " << f.second);
     auto testee = sys.spawn(f.first, &had_timeout, sys.spawn<lazy_init>(pong));
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"ping"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "ping"s);
     sched.run_once();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"pong"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "pong"s);
     sched.trigger_timeout();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 2u);
     // now, the timeout message is already dispatched, while pong did
@@ -276,11 +254,11 @@ CAF_TEST(nested_timeout) {
     sched.run();
     // dispatch second timeout
     CAF_REQUIRE_EQUAL(sched.trigger_timeout(), true);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"ping"});
-    CAF_CHECK(!had_timeout);
-    CAF_CHECK(sched.next_job<ping_actor>().state.had_first_timeout);
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "ping"s);
+    CHECK(!had_timeout);
+    CHECK(sched.next_job<ping_actor>().state.had_first_timeout);
     sched.run();
-    CAF_CHECK(had_timeout);
+    CHECK(had_timeout);
   }
 }
 
@@ -290,20 +268,20 @@ CAF_TEST(multiplexed_timeout) {
               {ping_multiplexed3, "ping_multiplexed3"}};
   for (auto f : fs) {
     bool had_timeout = false;
-    CAF_MESSAGE("test implemenation " << f.second);
+    MESSAGE("test implementation " << f.second);
     auto testee = sys.spawn(f.first, &had_timeout, sys.spawn<lazy_init>(pong));
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"ping"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "ping"s);
     sched.run_once();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 1u);
-    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), string{"pong"});
+    CAF_REQUIRE_EQUAL(sched.next_job<local_actor>().name(), "pong"s);
     sched.trigger_timeouts();
     CAF_REQUIRE_EQUAL(sched.jobs.size(), 2u);
     // now, the timeout message is already dispatched, while pong did
     // not respond to the message yet, i.e., timeout arrives before response
     sched.run();
-    CAF_CHECK(had_timeout);
+    CHECK(had_timeout);
   }
 }
 
-CAF_TEST_FIXTURE_SCOPE_END()
+END_FIXTURE_SCOPE()
