@@ -1,35 +1,21 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
-
-#include "caf/config.hpp"
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #define CAF_SUITE sum_type
-#include "caf/test/unit_test.hpp"
 
-#include <new>
+#include "caf/sum_type.hpp"
+
+#include "core-test.hpp"
+
 #include <map>
+#include <new>
 #include <string>
 
 #include "caf/default_sum_type_access.hpp"
 #include "caf/detail/overload.hpp"
 #include "caf/raise_error.hpp"
 #include "caf/static_visitor.hpp"
-#include "caf/sum_type.hpp"
 #include "caf/sum_type_access.hpp"
 
 namespace {
@@ -120,9 +106,12 @@ private:
   template <class Result, class Visitor, class... Ts>
   inline Result apply(Visitor&& f, Ts&&... xs) const {
     switch (index_) {
-      case 0: return f(std::forward<Ts>(xs)..., v0);
-      case 1: return f(std::forward<Ts>(xs)..., v1);
-      case 2: return f(std::forward<Ts>(xs)..., v2);
+      case 0:
+        return f(std::forward<Ts>(xs)..., v0);
+      case 1:
+        return f(std::forward<Ts>(xs)..., v1);
+      case 2:
+        return f(std::forward<Ts>(xs)..., v2);
     }
     CAF_RAISE_ERROR("invalid index in union_type");
   }
@@ -188,70 +177,70 @@ constexpr stringify_t stringify = stringify_t{};
 
 CAF_TEST(holds_alternative) {
   union_type x;
-  CAF_CHECK_EQUAL(holds_alternative<int>(x), true);
-  CAF_CHECK_EQUAL(holds_alternative<string>(x), false);
-  CAF_CHECK_EQUAL(holds_alternative<map_type>(x), false);
+  CHECK_EQ(holds_alternative<int>(x), true);
+  CHECK_EQ(holds_alternative<string>(x), false);
+  CHECK_EQ(holds_alternative<map_type>(x), false);
   x = string{"hello world"};
-  CAF_CHECK_EQUAL(holds_alternative<int>(x), false);
-  CAF_CHECK_EQUAL(holds_alternative<string>(x), true);
-  CAF_CHECK_EQUAL(holds_alternative<map_type>(x), false);
+  CHECK_EQ(holds_alternative<int>(x), false);
+  CHECK_EQ(holds_alternative<string>(x), true);
+  CHECK_EQ(holds_alternative<map_type>(x), false);
   x = map_type{{1, 1}, {2, 2}};
-  CAF_CHECK_EQUAL(holds_alternative<int>(x), false);
-  CAF_CHECK_EQUAL(holds_alternative<string>(x), false);
-  CAF_CHECK_EQUAL(holds_alternative<map_type>(x), true);
+  CHECK_EQ(holds_alternative<int>(x), false);
+  CHECK_EQ(holds_alternative<string>(x), false);
+  CHECK_EQ(holds_alternative<map_type>(x), true);
 }
 
 CAF_TEST(get) {
   union_type x;
-  CAF_CHECK_EQUAL(get<int>(x), 0);
+  CHECK_EQ(get<int>(x), 0);
   x = 42;
-  CAF_CHECK_EQUAL(get<int>(x), 42);
+  CHECK_EQ(get<int>(x), 42);
   x = string{"hello world"};
-  CAF_CHECK_EQUAL(get<string>(x), "hello world");
+  CHECK_EQ(get<string>(x), "hello world");
   x = map_type{{1, 1}, {2, 2}};
-  CAF_CHECK_EQUAL(get<map_type>(x), map_type({{1, 1}, {2, 2}}));
+  CHECK_EQ(get<map_type>(x), map_type({{1, 1}, {2, 2}}));
 }
 
 CAF_TEST(get_if) {
   union_type x;
-  CAF_CHECK_EQUAL(get_if<int>(&x), &get<int>(x));
-  CAF_CHECK_EQUAL(get_if<string>(&x), nullptr);
-  CAF_CHECK_EQUAL(get_if<map_type>(&x), nullptr);
+  CHECK_EQ(get_if<int>(&x), &get<int>(x));
+  CHECK_EQ(get_if<string>(&x), nullptr);
+  CHECK_EQ(get_if<map_type>(&x), nullptr);
   x = string{"hello world"};
-  CAF_CHECK_EQUAL(get_if<int>(&x), nullptr);
-  CAF_CHECK_EQUAL(get_if<string>(&x), &get<string>(x));
-  CAF_CHECK_EQUAL(get_if<map_type>(&x), nullptr);
+  CHECK_EQ(get_if<int>(&x), nullptr);
+  CHECK_EQ(get_if<string>(&x), &get<string>(x));
+  CHECK_EQ(get_if<map_type>(&x), nullptr);
   x = map_type{{1, 1}, {2, 2}};
-  CAF_CHECK_EQUAL(get_if<int>(&x), nullptr);
-  CAF_CHECK_EQUAL(get_if<string>(&x), nullptr);
-  CAF_CHECK_EQUAL(get_if<map_type>(&x), &get<map_type>(x));
+  CHECK_EQ(get_if<int>(&x), nullptr);
+  CHECK_EQ(get_if<string>(&x), nullptr);
+  CHECK_EQ(get_if<map_type>(&x), &get<map_type>(x));
 }
 
 CAF_TEST(unary visit) {
   union_type x;
-  CAF_CHECK_EQUAL(visit(stringify, x), "0");
+  CHECK_EQ(visit(stringify, x), "0");
   x = string{"hello world"};
-  CAF_CHECK_EQUAL(visit(stringify, x), "hello world");
+  CHECK_EQ(visit(stringify, x), "hello world");
   x = map_type{{1, 1}, {2, 2}};
-  CAF_CHECK_EQUAL(visit(stringify, x), "{1 = 1, 2 = 2}");
+  CHECK_EQ(visit(stringify, x), "{1 = 1, 2 = 2}");
 }
 
 CAF_TEST(binary visit) {
   union_type x;
   union_type y;
-  CAF_CHECK_EQUAL(visit(stringify, x, y), "0, 0");
+  CHECK_EQ(visit(stringify, x, y), "0, 0");
   x = 42;
   y = string{"hello world"};
-  CAF_CHECK_EQUAL(visit(stringify, x, y), "42, hello world");
+  CHECK_EQ(visit(stringify, x, y), "42, hello world");
 }
 
 CAF_TEST(ternary visit) {
   union_type x;
   union_type y;
   union_type z;
-  //CAF_CHECK_EQUAL(visit(stringify, x, y, z), "0, 0, 0");
+  // CHECK_EQ(visit(stringify, x, y, z), "0, 0, 0");
   x = 42;
   y = string{"foo"};
   z = map_type{{1, 1}, {2, 2}};
-  CAF_CHECK_EQUAL(visit(stringify, x, y, z), "42, foo, {1 = 1, 2 = 2}");
+  CHECK_EQ(visit(stringify, x, y, z), "42, foo, {1 = 1, 2 = 2}");
 }
