@@ -1,20 +1,6 @@
-/******************************************************************************
- *                       ____    _    _____                                   *
- *                      / ___|  / \  |  ___|    C++                           *
- *                     | |     / _ \ | |_       Actor                         *
- *                     | |___ / ___ \|  _|      Framework                     *
- *                      \____/_/   \_|_|                                      *
- *                                                                            *
- * Copyright 2011-2018 Dominik Charousset                                     *
- *                                                                            *
- * Distributed under the terms and conditions of the BSD 3-Clause License or  *
- * (at your option) under the terms and conditions of the Boost Software      *
- * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
- *                                                                            *
- * If you did not receive a copy of the license files, see                    *
- * http://opensource.org/licenses/BSD-3-Clause and                            *
- * http://www.boost.org/LICENSE_1_0.txt.                                      *
- ******************************************************************************/
+// This file is part of CAF, the C++ Actor Framework. See the file LICENSE in
+// the main distribution directory for license terms and copyright or visit
+// https://github.com/actor-framework/actor-framework/blob/master/LICENSE.
 
 #include "caf/config.hpp"
 
@@ -29,56 +15,30 @@
 
 #  include "caf/all.hpp"
 
-#  define ERROR_HANDLER [&](error& err) { CAF_FAIL(system.render(err)); }
-
-namespace {
-
-using server_type = caf::typed_actor<caf::replies_to<my_request>::with<bool>>;
-
-using event_testee_type
-  = caf::typed_actor<caf::replies_to<get_state_atom>::with<std::string>,
-                     caf::replies_to<std::string>::with<void>,
-                     caf::replies_to<float>::with<void>,
-                     caf::replies_to<int32_t>::with<int32_t>>;
-
-using string_actor
-  = caf::typed_actor<caf::replies_to<std::string>::with<std::string>>;
-
-using int_actor = caf::typed_actor<caf::replies_to<int32_t>::with<int32_t>>;
-
-using float_actor = caf::typed_actor<caf::reacts_to<float>>;
-
-} // namespace
+#  define ERROR_HANDLER [&](error& err) { CAF_FAIL(err); }
 
 using std::string;
 
 using namespace caf;
-
-using passed_atom = caf::atom_constant<caf::atom("passed")>;
-
-constexpr auto passed_atom_v = passed_atom::value;
+using namespace std::string_literals;
 
 namespace {
 
-enum class mock_errc : uint8_t {
-  cannot_revert_empty = 1,
-};
+static_assert(std::is_same<reacts_to<int, int>, result<void>(int, int)>::value);
 
-error make_error(mock_errc x) {
-  return {static_cast<uint8_t>(x), atom("mock")};
-}
+static_assert(std::is_same<replies_to<double>::with<double>,
+                           result<double>(double)>::value);
 
 // check invariants of type system
-using dummy1
-  = typed_actor<reacts_to<int32_t, int32_t>, replies_to<double>::with<double>>;
+using dummy1 = typed_actor<result<void>(int, int), result<double>(double)>;
 
 using dummy2 = dummy1::extend<reacts_to<ok_atom>>;
 
 static_assert(std::is_convertible<dummy2, dummy1>::value,
               "handle not assignable to narrower definition");
 
-using dummy3 = typed_actor<reacts_to<float, int32_t>>;
-using dummy4 = typed_actor<replies_to<int32_t>::with<double>>;
+using dummy3 = typed_actor<reacts_to<float, int>>;
+using dummy4 = typed_actor<replies_to<int>::with<double>>;
 using dummy5 = dummy4::extend_with<dummy3>;
 
 static_assert(std::is_convertible<dummy5, dummy3>::value,
@@ -91,6 +51,8 @@ static_assert(std::is_convertible<dummy5, dummy4>::value,
  *                        simple request/response test                        *
  ******************************************************************************/
 
+using server_type = typed_actor<replies_to<my_request>::with<bool>>;
+
 server_type::behavior_type typed_server1() {
   return {
     [](const my_request& req) { return req.a == req.b; },
@@ -116,10 +78,10 @@ public:
 void client(event_based_actor* self, const actor& parent,
             const server_type& serv) {
   self->request(serv, infinite, my_request{0, 0}).then([=](bool val1) {
-    CAF_CHECK_EQUAL(val1, true);
+    CHECK_EQ(val1, true);
     self->request(serv, infinite, my_request{10, 20}).then([=](bool val2) {
-      CAF_CHECK_EQUAL(val2, false);
-      self->send(parent, passed_atom_v);
+      CHECK_EQ(val2, false);
+      self->send(parent, ok_atom_v);
     });
   });
 }
@@ -128,39 +90,41 @@ void client(event_based_actor* self, const actor& parent,
  *          test skipping of messages intentionally + using become()          *
  ******************************************************************************/
 
+using event_testee_type
+  = typed_actor<replies_to<get_state_atom>::with<string>,
+                replies_to<string>::with<void>, replies_to<float>::with<void>,
+                replies_to<int>::with<int>>;
+
 class event_testee : public event_testee_type::base {
 public:
   event_testee(actor_config& cfg) : event_testee_type::base(cfg) {
-    // nop
+    set_default_handler(skip);
   }
 
   behavior_type wait4string() {
     return {
+      partial_behavior_init,
       [=](get_state_atom) { return "wait4string"; },
       [=](const string&) { become(wait4int()); },
-      [=](float) { return skip(); },
-      [=](int) { return skip(); },
     };
   }
 
   behavior_type wait4int() {
     return {
+      partial_behavior_init,
       [=](get_state_atom) { return "wait4int"; },
       [=](int) -> int {
         become(wait4float());
         return 42;
       },
-      [=](float) { return skip(); },
-      [=](const string&) { return skip(); },
     };
   }
 
   behavior_type wait4float() {
     return {
+      partial_behavior_init,
       [=](get_state_atom) { return "wait4float"; },
       [=](float) { become(wait4string()); },
-      [=](const string&) { return skip(); },
-      [=](int) { return skip(); },
     };
   }
 
@@ -173,6 +137,8 @@ public:
  *                         simple 'forwarding' chain                          *
  ******************************************************************************/
 
+using string_actor = typed_actor<replies_to<string>::with<string>>;
+
 string_actor::behavior_type string_reverter() {
   return {
     [](string& str) -> string {
@@ -201,9 +167,9 @@ maybe_string_actor::behavior_type maybe_string_reverter() {
   return {
     [](string& str) -> result<ok_atom, string> {
       if (str.empty())
-        return mock_errc::cannot_revert_empty;
+        return sec::invalid_argument;
       std::reverse(str.begin(), str.end());
-      return {ok_atom::value, std::move(str)};
+      return {ok_atom_v, std::move(str)};
     },
   };
 }
@@ -240,7 +206,7 @@ behavior foo(event_based_actor* self) {
 
 int_actor::behavior_type int_fun2(int_actor::pointer self) {
   self->set_down_handler([=](down_msg& dm) {
-    CAF_CHECK_EQUAL(dm.reason, exit_reason::normal);
+    CHECK_EQ(dm.reason, exit_reason::normal);
     self->quit();
   });
   return {
@@ -263,7 +229,7 @@ behavior foo2(event_based_actor* self) {
 float_actor::behavior_type float_fun(float_actor::pointer self) {
   return {
     [=](float a) {
-      CAF_CHECK_EQUAL(a, 1.0f);
+      CHECK_EQ(a, 1.0f);
       self->quit(exit_reason::user_shutdown);
     },
   };
@@ -277,156 +243,163 @@ int_actor::behavior_type foo3(int_actor::pointer self) {
   };
 }
 
-struct fixture {
-  actor_system_config cfg;
-  actor_system system;
-  scoped_actor self;
-
-  static actor_system_config& init(actor_system_config& cfg) {
-    cfg.add_message_types<id_block::core_test>();
-    cfg.parse(test::engine::argc(), test::engine::argv());
-    return cfg;
-  }
-
-  fixture() : system(init(cfg)), self(system) {
-    // nop
-  }
-
+struct fixture : test_coordinator_fixture<> {
   void test_typed_spawn(server_type ts) {
-    self->send(ts, my_request{1, 2});
-    self->receive([](bool value) { CAF_CHECK_EQUAL(value, false); });
-    CAF_MESSAGE("async send + receive");
-    self->send(ts, my_request{42, 42});
-    self->receive([](bool value) { CAF_CHECK_EQUAL(value, true); });
-    CAF_MESSAGE("request + receive with result true");
-    self->request(ts, infinite, my_request{10, 20})
-      .receive([](bool value) { CAF_CHECK_EQUAL(value, false); },
-               ERROR_HANDLER);
-    CAF_MESSAGE("request + receive with result false");
-    self->request(ts, infinite, my_request{0, 0})
-      .receive([](bool value) { CAF_CHECK_EQUAL(value, true); }, ERROR_HANDLER);
-    CAF_CHECK_EQUAL(system.registry().running(), 2u);
+    MESSAGE("the server returns false for inequal numbers");
+    inject((my_request), from(self).to(ts).with(my_request{1, 2}));
+    expect((bool), from(ts).to(self).with(false));
+    MESSAGE("the server returns true for equal numbers");
+    inject((my_request), from(self).to(ts).with(my_request{42, 42}));
+    expect((bool), from(ts).to(self).with(true));
+    CHECK_EQ(sys.registry().running(), 2u);
     auto c1 = self->spawn(client, self, ts);
-    self->receive([](passed_atom) { CAF_MESSAGE("received `passed_atom`"); });
-    self->wait_for(c1);
-    CAF_CHECK_EQUAL(system.registry().running(), 2u);
+    run();
+    expect((ok_atom), from(c1).to(self).with(ok_atom_v));
+    CHECK_EQ(sys.registry().running(), 2u);
   }
 };
 
 } // namespace
 
-CAF_TEST_FIXTURE_SCOPE(typed_spawn_tests, fixture)
+BEGIN_FIXTURE_SCOPE(fixture)
 
 /******************************************************************************
  *                             put it all together                            *
  ******************************************************************************/
 
 CAF_TEST(typed_spawns) {
-  CAF_MESSAGE("run test series with typed_server1");
-  test_typed_spawn(system.spawn(typed_server1));
+  MESSAGE("run test series with typed_server1");
+  test_typed_spawn(sys.spawn(typed_server1));
   self->await_all_other_actors_done();
-  CAF_MESSAGE("finished test series with `typed_server1`");
-  CAF_MESSAGE("run test series with typed_server2");
-  test_typed_spawn(system.spawn(typed_server2));
+  MESSAGE("finished test series with `typed_server1`");
+  MESSAGE("run test series with typed_server2");
+  test_typed_spawn(sys.spawn(typed_server2));
   self->await_all_other_actors_done();
-  CAF_MESSAGE("finished test series with `typed_server2`");
-  test_typed_spawn(self->spawn<typed_server3>("hi there", self));
-  self->receive([](const string& str) { CAF_REQUIRE_EQUAL(str, "hi there"); });
+  MESSAGE("finished test series with `typed_server2`");
+  auto serv3 = self->spawn<typed_server3>("hi there", self);
+  run();
+  expect((string), from(serv3).to(self).with("hi there"s));
+  test_typed_spawn(serv3);
 }
 
 CAF_TEST(event_testee_series) {
   auto et = self->spawn<event_testee>();
-  string result;
+  MESSAGE("et->message_types() returns an interface description");
+  typed_actor<replies_to<get_state_atom>::with<string>> sub_et = et;
+  std::set<string> iface{"(get_state_atom) -> (std::string)",
+                         "(std::string) -> (void)", "(float) -> (void)",
+                         "(int32_t) -> (int32_t)"};
+  CHECK_EQ(join(sub_et->message_types(), ","), join(iface, ","));
+  MESSAGE("the testee skips messages to drive its internal state machine");
   self->send(et, 1);
   self->send(et, 2);
   self->send(et, 3);
   self->send(et, .1f);
-  self->send(et, "hello event testee!");
+  self->send(et, "hello event testee!"s);
   self->send(et, .2f);
   self->send(et, .3f);
-  self->send(et, "hello again event testee!");
-  self->send(et, "goodbye event testee!");
-  typed_actor<replies_to<get_state_atom>::with<string>> sub_et = et;
-  std::set<string> iface{"caf::replies_to<get_state_atom>::with<@str>",
-                         "caf::replies_to<@str>::with<void>",
-                         "caf::replies_to<float>::with<void>",
-                         "caf::replies_to<@i32>::with<@i32>"};
-  CAF_CHECK_EQUAL(join(sub_et->message_types(), ","), join(iface, ","));
-  self->send(sub_et, get_state_atom_v);
-  // we expect three 42s
-  int i = 0;
-  self->receive_for(i, 3)([](int value) { CAF_CHECK_EQUAL(value, 42); });
-  self->receive([&](const string& str) { result = str; },
-                after(std::chrono::minutes(1)) >>
-                  [&] { CAF_FAIL("event_testee does not reply"); });
-  CAF_CHECK_EQUAL(result, "wait4int");
+  self->send(et, "hello again event testee!"s);
+  self->send(et, "goodbye event testee!"s);
+  run();
+  expect((int), from(et).to(self).with(42));
+  expect((int), from(et).to(self).with(42));
+  expect((int), from(et).to(self).with(42));
+  inject((get_state_atom), from(self).to(sub_et).with(get_state_atom_v));
+  expect((string), from(et).to(self).with("wait4int"s));
 }
 
 CAF_TEST(string_delegator_chain) {
   // run test series with string reverter
   auto aut = self->spawn<monitored>(string_delegator,
-                                    system.spawn(string_reverter), true);
-  std::set<string> iface{"caf::replies_to<@str>::with<@str>"};
-  CAF_CHECK_EQUAL(aut->message_types(), iface);
-  self->request(aut, infinite, "Hello World!")
-    .receive(
-      [](const string& answer) { CAF_CHECK_EQUAL(answer, "!dlroW olleH"); },
-      ERROR_HANDLER);
+                                    sys.spawn(string_reverter), true);
+  std::set<string> iface{"(std::string) -> (std::string)"};
+  CHECK_EQ(aut->message_types(), iface);
+  inject((string), from(self).to(aut).with("Hello World!"s));
+  run();
+  expect((string), to(self).with("!dlroW olleH"s));
 }
 
 CAF_TEST(maybe_string_delegator_chain) {
   CAF_LOG_TRACE(CAF_ARG(self));
-  auto aut = system.spawn(maybe_string_delegator,
-                          system.spawn(maybe_string_reverter));
-  CAF_MESSAGE("send empty string, expect error");
-  self->request(aut, infinite, "")
-    .receive(
-      [](ok_atom, const string&) { CAF_FAIL("unexpected string response"); },
-      [](const error& err) {
-        CAF_CHECK_EQUAL(err.category(), atom("mock"));
-        CAF_CHECK_EQUAL(err.code(),
-                        static_cast<uint8_t>(mock_errc::cannot_revert_empty));
-      });
-  CAF_MESSAGE("send abcd string, expect dcba");
-  self->request(aut, infinite, "abcd")
-    .receive([](ok_atom, const string& str) { CAF_CHECK_EQUAL(str, "dcba"); },
-             ERROR_HANDLER);
+  auto aut = sys.spawn(maybe_string_delegator,
+                       sys.spawn(maybe_string_reverter));
+  MESSAGE("send empty string, expect error");
+  inject((string), from(self).to(aut).with(""s));
+  run();
+  expect((error), to(self).with(sec::invalid_argument));
+  MESSAGE("send abcd string, expect dcba");
+  inject((string), from(self).to(aut).with("abcd"s));
+  run();
+  expect((ok_atom, string), to(self).with(ok_atom_v, "dcba"s));
 }
 
 CAF_TEST(sending_typed_actors) {
-  auto aut = system.spawn(int_fun);
+  auto aut = sys.spawn(int_fun);
   self->send(self->spawn(foo), 10, aut);
-  self->receive([](int i) { CAF_CHECK_EQUAL(i, 100); });
+  run();
+  expect((int), to(self).with(100));
   self->spawn(foo3);
+  run();
 }
 
 CAF_TEST(sending_typed_actors_and_down_msg) {
-  auto aut = system.spawn(int_fun2);
+  auto aut = sys.spawn(int_fun2);
   self->send(self->spawn(foo2), 10, aut);
-  self->receive([](int i) { CAF_CHECK_EQUAL(i, 100); });
+  run();
+  expect((int), to(self).with(100));
 }
 
 CAF_TEST(check_signature) {
   using foo_type = typed_actor<replies_to<put_atom>::with<ok_atom>>;
-  using foo_result_type = optional<ok_atom>;
+  using foo_result_type = result<ok_atom>;
   using bar_type = typed_actor<reacts_to<ok_atom>>;
   auto foo_action = [](foo_type::pointer ptr) -> foo_type::behavior_type {
     return {
       [=](put_atom) -> foo_result_type {
         ptr->quit();
-        return {ok_atom::value};
+        return {ok_atom_v};
       },
     };
   };
   auto bar_action = [=](bar_type::pointer ptr) -> bar_type::behavior_type {
     auto foo = ptr->spawn<linked>(foo_action);
-    ptr->send(foo, put_atom::value);
-    return {[=](ok_atom) { ptr->quit(); }};
+    ptr->send(foo, put_atom_v);
+    return {
+      [=](ok_atom) { ptr->quit(); },
+    };
   };
   auto x = self->spawn(bar_action);
-  self->wait_for(x);
+  run();
 }
 
-CAF_TEST_FIXTURE_SCOPE_END()
+SCENARIO("state classes may use typed pointers") {
+  GIVEN("a state class for a statically typed actor type") {
+    using foo_type = typed_actor<result<int32_t>(get_atom)>;
+    struct foo_state {
+      foo_state(foo_type::pointer_view selfptr) : self(selfptr) {
+        foo_type hdl{self};
+        CHECK_EQ(selfptr, actor_cast<abstract_actor*>(hdl));
+        foo_type hdl2{selfptr};
+        CHECK_EQ(hdl, hdl2);
+      }
+      foo_type::behavior_type make_behavior() {
+        return {
+          [](get_atom) { return int32_t{42}; },
+        };
+      }
+      foo_type::pointer_view self;
+    };
+    using foo_impl = stateful_actor<foo_state, foo_type::impl>;
+    WHEN("spawning a stateful actor using the state class") {
+      auto foo = sys.spawn<foo_impl>();
+      THEN("the actor calls make_behavior of the state class") {
+        inject((get_atom), from(self).to(foo).with(get_atom_v));
+        expect((int32_t), from(foo).to(self).with(42));
+      }
+    }
+  }
+}
+
+END_FIXTURE_SCOPE()
 
 #endif // CAF_WINDOWS
